//
// bop.h
//
// Written by:  Yanting Wang   Sep. 27, 2001
//
// Bond Order Parameters used to distinguish the crystal and liquid states.
//

#ifndef __BOP__H
#define __BOP__H

#include "numtools.h"
#include "simulate.h"

#include <fstream>
#include <iostream>

using namespace std;

//
// Calculate the Bond Order Parameter of a given conformation.
//

class BOP
{
private:
  Particle **const &p;
  long particleNum;

  double rb;      // largest distance that particles can be near neighbors

private:
  long bondNum;    // number of bonds
  double *cosTheta, *phi;     // bonds

private: 
  // calculate bonds and put them into bonds arrays
  void getBonds( int surface, long *p1, long *p2, 
		 double *x, double *y, double *z, 
		 double *r, long pn);  

public:
  BOP( Particle **const &p1, long pn, double rb1 );
  ~BOP();

  // read conformation from an input stream
  static void readConf( ifstream &iFile, Particle **&p, long &num );

  // calculate bond order parameters w[l]
  // Q[l] is acompanyingly calculated
  double w( int l, int surface, long *p1, long *p2, 
	    double *x, double *y, double *z, double *r, long pn, double *q );
};

#endif //__BOP_H
