//
// numtools.C
//
// Written by:  Yanting Wang   September 26, 2001
//
// Some numerical functions.
//

#include "numtools.h"

#include <iostream>
using namespace std;

#include <stdlib.h>

complex complex::operator+( const complex &c2 ) const
{
  return complex( r + c2.r, i + c2.i );
} 

complex complex::operator-( const complex &c2 ) const
{
  return complex( r - c2.r, i - c2.i );
}

complex complex::operator*( double c ) const
{
  return complex( c * r, c * i );
}

complex complex::operator/( double c ) const
{
  return complex( r / c, i / c );
}

complex complex::operator+=( const complex &c2 )
{
  *this = *this + c2;

  return *this;
}

complex complex::operator-=( const complex &c2 )
{
  *this = *this - c2;

  return *this;
}

complex complex::operator*=( double c )
{
  *this = *this * c;

  return *this;
}

complex complex::operator/=( double c )
{
  *this = *this / c;

  return *this;
}

complex complex::operator*( const complex &c2 ) const
{
  return complex( r*c2.r - i*c2.i, r*c2.i + i*c2.r );
}

complex complex::operator*=( const complex &c2 )
{
  *this = *this * c2;

  return *this;
}

const complex & complex::operator=( const complex &right )
{
  r = right.r;   i= right.i;

  return *this;   // enables cascading
}


const double Numeric::PI = 3.1415926536;

long Numeric::factorial( int n )
{
  if( n==0 ) return 1;
  else if( n==1 ) return 1;
  else if( n==2 ) return 2;
  else if( n==3 ) return 6;
  else if( n==4 ) return 24;
  else if( n==5 ) return 120;
  else if( n==6 ) return 720;
  else if( n<0 )
  {
    long f=1;
    for( long i=-1; i>=n; i-- ) f *= i;
    return f;
  }    
  else
  { 
    long f = 720;
    for( long i=7; i<=n; i++ ) f *= i;
    return f;
  }
}

//
// Computes the associated Legendre polynomial P(l,m,x). Here m and l are
// integers satisfying 0<=m<=l, while x lies in the range -1<=x<=1.
//
// Modified a little from Numerical Recipes in C ( 2nd Edition ) Page 254
//

double Numeric::legendre( int l, int m, double x )
{
  double fact, pll, pmm, pmmp1, somx2;
  int i,ll;

  if( m < 0 || m > l || fabs(x) > 1.0 )
  {
    cerr << "Bad arguments in routine Numeric::legendre." << endl;
    cerr << "m l x : " << m << " " << l << " " << x << endl;
    exit(1);
  }

  pmm = 1.0;     // compute p( m, m )

  if( m > 0 )
  {
    somx2 = sqrt( ( 1.0 - x ) * ( 1.0 + x ) );
    fact = 1.0;
    for( i=1; i<=m; i++ )
    {
      pmm *= -fact * somx2;
      fact += 2.0;
    }
  }

  if( l == m )   return pmm;
  else           // compute p( m, m+1 )
  {
    pmmp1 = x * ( 2*m + 1 ) * pmm;
    if( l == ( m + 1 ) )   return pmmp1;
    else
    {
      for( ll=m+2; ll<=l; ll++ )
      {
	pll = ( x * ( 2*ll -1 ) * pmmp1 - ( ll + m - 1 ) * pmm ) / ( ll - m );
	pmm = pmmp1;
	pmmp1 = pll;
      }
      return pll;
    }
  }
}

complex Numeric::sphHarmonic( int l, int m, double cosTheta, double phi )
{
  int m1 = abs( m );

  double c = sqrt( ( 2*l +1 ) * factorial( l-m1 ) / 
		   ( 4 * PI * factorial( l+m1 ) ) );

  c *= legendre( l, m1, cosTheta );

  complex y;

  if( m<0 ) 
    y = complex( cos( m1*phi ), -sin( m1*phi ) ) * ( (m&1) ? -1.0 : 1.0 );
  else 
    y = complex( cos( m1*phi ), sin( m1*phi ) );

  return y * c;
}
