//
// numtools.h
//
// Written by : Yanting Wang   September 26, 2001
//
// Set of useful numerical functions.
//

#ifndef __NUMTOOL_H
#define __NUMTOOL_H

#include <math.h>

struct complex
{
  double r;
  double i;

  complex( double r1 = 0.0, double i1 = 0.0 ) : r( r1 ), i( i1 ) {}

  double a() const { return sqrt( r*r + i*i ); }   // get the length
  double a2() const { return r*r + i*i; }

  complex operator+( const complex &c2 ) const;
  complex operator-( const complex &c2 ) const;
  complex operator*( double c ) const;
  complex operator/( double c ) const;
  complex operator+=( const complex &c2 );
  complex operator-=( const complex &c2 );
  complex operator*=( double c );
  complex operator/=( double c );
  complex operator*( const complex &c2 ) const;
  complex operator*=( const complex &c2 );
  const complex &operator=( const complex & );
};

class Numeric
{
public:
  static const double PI;

public:
  static bool fzero( double x ) { return fabs( x ) < 1.0e-30; }
  static long factorial( int n );

public:
  static double legendre( int l, int m, double x );

  // spherical harmonic function
  // note cos( theta ) is used for possible computing time saving
  static complex sphHarmonic( int l, int m, double cosTheta, double phi );  

  // Wigner 3-j symbols in threejm.C
  static void ThreeJSymbolM( double l1, double l2, double l3, double m1, 
			     double &m2min, double &m2max, double *thrcof, 
			     int ndim, int &errflag );
};

#endif //__NUMTOOL_H
