//
// vector.C
//
// Move global reloaded operators for class Vector here to avoid compiling
// confliction.
//
// Written by: Yanting Wang             12/21/2005
//

#include "vector.h"

#include <iostream>
using std::cout;
using std::endl;

Vector operator+( const Vector &v1, const Vector &v2 )
{
  return Vector( v1.x + v2.x, v1.y + v2.y, v1.z + v2.z );
}

Vector operator-( const Vector &v1, const Vector &v2 )
{
  return Vector( v1.x - v2.x, v1.y - v2.y, v1.z - v2.z );
}

Vector operator-( const Vector &v1 )
{
  return Vector( -v1.x, -v1.y, -v1.z );
}

Vector operator*( const Vector &v, const double d )
{
  return Vector( v.x*d, v.y*d, v.z*d );
}

Vector operator*( const double d, const Vector &v )
{
  return Vector( v.x*d, v.y*d, v.z*d );
}

//scalar product
double operator*( const Vector &v1, const Vector &v2 )
{
  return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;
}

//vector product
Vector operator^( const Vector &v1, const Vector &v2 )
{
  return Vector( v1.y*v2.z - v1.z*v2.y,
		 v1.z*v2.x - v1.x*v2.z,
		 v1.x*v2.y - v1.y*v2.x );
}

Vector operator/( const Vector &v, const double d )
{
  return Vector( v.x/d, v.y/d, v.z/d );
}

double costheta( const Vector &v1, const Vector &v2 )
{
  return (v1.x*v2.x + v1.y*v2.y + v1.z*v2.z) /
    sqrt( v1.x * v1.x + v1.y * v1.y + v1.z * v1.z ) /
    sqrt( v2.x * v2.x + v2.y * v2.y + v2.z * v2.z );

  // return v1 * v2 / v1.r() / v2.r();
}

ostream &operator<<( ostream &output, const Vector &p )
{
  output << p.x << " " << p.y << " " << p.z;
  return output;
}

istream &operator>>( istream &input, Vector &p )
{
  input >> p.x >> p.y >> p.z;
  return input;
}

