//
// vector.h
//
// Vector structure and its operations for 3-d space vectors.
//
// Written by: Yanting Wang           12/25/2002
//

#include <iostream>
using std::istream;
using std::ostream;

#include <math.h>

#ifndef __VECTOR__H
#define __VECTOR__H

struct Vector
{
  double x, y, z;

  Vector() : x(0.0), y(0.0), z(0.0) {}
  Vector( double x, double y, double z ) : x(x), y(y), z(z) {}

  void set( double xx, double yy, double zz ) { x = xx; y = yy; z = zz; }

  void clear() { x = 0.0;   y = 0.0;   z = 0.0; }

  // square length of a vector
  double r2() const { return x*x + y*y + z*z; }

  // length of a vector
  double r() const { return sqrt( x*x + y*y + z*z ); }

  // the unit vector
  Vector normal() const
  {
    double r = sqrt( x*x + y*y + z*z );
    return Vector( x/r, y/r, z/r );
  }

  const Vector &operator = ( const Vector &v ) 
  { 
    x = v.x;   y = v.y;   z = v.z; 
    return v;
  }

  const Vector &operator += ( const Vector &v )
  {
    x += v.x;   y += v.y;   z += v.z;
    return *this;
  }

  const Vector &operator -= ( const Vector &v )
  {
    x -= v.x;   y -= v.y;   z -= v.z;
    return *this;
  }

  const Vector &operator *= ( double d )
  {
    x *= d;   y *= d;   z *= d;
    return *this;
  }

  const Vector &operator /= ( double d )
  {
    x /= d;   y /= d;   z /= d;
    return *this;
  }
};

Vector operator+( const Vector &v1, const Vector &v2 );
Vector operator-( const Vector &v1, const Vector &v2 );
Vector operator-( const Vector &v1 );
Vector operator*( const Vector &v, const double d );
Vector operator*( const double d, const Vector &v );
//scalar product
double operator*( const Vector &v1, const Vector &v2 );
//vector product
Vector operator^( const Vector &v1, const Vector &v2 );
Vector operator/( const Vector &v, const double d );
// cosine of the angle between two vectors
double costheta( const Vector &v1, const Vector &v2 );
ostream &operator<<( ostream &output, const Vector &p );
istream &operator>>( istream &input, Vector &p );

#endif //__VECTOR__H
